@@ -603,11 +603,9 @@ _domScanHashForRefs(xmlHashTablePtr self) {
     return refs;
 }
 
-// Determine if there's any API references to the node or its descendants
-DLLEXPORT int
-domNodeIsReferenced(xmlNodePtr self) {
-    xmlAttrPtr attr;
-    xmlNodePtr cur;
+// Determine if there's any API references to a node or its descendants
+static int
+_domItemIsReferenced(xmlNodePtr self) {
     assert(self != NULL);
 
     // cheap checks
@@ -622,13 +620,8 @@ domNodeIsReferenced(xmlNodePtr self) {
 
     if (self->type == XML_ELEMENT_NODE) {
         // scan element attributes
-        for (attr = self->properties; attr != NULL; attr = attr->next) {
-            void* ref = attr->type == XML_NAMESPACE_DECL
-                ? ((xmlNsPtr)attr)->_private
-                : attr->_private;
-            if (ref != NULL) {
-                return 1;
-            }
+        if (domNodeIsReferenced((xmlNodePtr)self->properties)) {
+            return 1;
         }
     }
     else if (self->type == XML_DTD_NODE) {
@@ -642,7 +635,7 @@ domNodeIsReferenced(xmlNodePtr self) {
     else if (_domIsDoc(self)) {
         // check external DtD
         xmlDtdPtr dtd = ((xmlDocPtr) self)->extSubset;
-        if (dtd != NULL && domNodeIsReferenced((xmlNodePtr)dtd)) {
+        if (dtd != NULL && _domItemIsReferenced((xmlNodePtr)dtd)) {
             return 1;
         }
     }
@@ -655,19 +648,27 @@ domNodeIsReferenced(xmlNodePtr self) {
         }
     }
     else if (self->type != XML_ENTITY_REF_NODE) {
-        // scan siblings and children
-        for (cur = self; cur != NULL; cur = cur->next) {
-            xmlNodePtr kids = cur->children;
-            if (cur->_private != NULL
-                || (kids != NULL && domNodeIsReferenced(kids))) {
-                return 1;
-            }
+        // scan children
+        if (domNodeIsReferenced(self->children)) {
+            return 1;
         }
     }
 
     return 0;
 }
 
+// Determine if there's any API references in a node or its siblings
+DLLEXPORT int
+domNodeIsReferenced(xmlNodePtr cur) {
+    if (cur == NULL) return 0;
+    while (cur->prev) cur = cur->prev;
+    while (cur) {
+        if (_domItemIsReferenced(cur)) return 1;
+        cur = cur->next;
+    }
+    return 0;
+}
+
 DLLEXPORT void
 domReleaseNode( xmlNodePtr node ) {
     domUnlinkNode(node);
@@ -1458,11 +1459,6 @@ static int _domNamecmp(xmlNodePtr self, const xmlChar* pname) {
     int rv;
     xmlChar* name = domGetNodeName(self);
     rv = xmlStrcmp( name, pname );
-    if (rv) {
-        xmlFree(name);
-        name = domGetNodeName(self);
-        rv = xmlStrcmp( name, pname );
-    }
     xmlFree(name);
     return rv;
 }
